-- /psets/2-notes/notes-create.sql
--
-- Copyright (C) 1998-2007 various parties
--
-- This program is free software. You can redistribute it and/or modify it under
-- the terms of the GNU General Public License as published by the Free Software
-- Foundation; either version 2 of the License, or (at your option) any later version.
-- This program is distributed in the hope that it will be useful, but WITHOUT ANY
-- WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-- PARTICULAR PURPOSE. See the GNU General Public License for more details.
--
-- Edited by <frank.bergmann@project-open-com>



-----------------------------------------------------------
-- Create the OpenACS object type
--
-- The object types contains basic meta-data about an object.
SELECT acs_object_type__create_type (
	'note',				-- object_type
	'Note',				-- pretty_name
	'Notes',			-- pretty_plural
	'acs_object',			-- supertype
	'notes',			-- table_name
	'note_id',			-- id_column
	'notes',			-- package_name
	'f',				-- abstract_p
	null,				-- type_extension_table
	'note__name'			-- name_method
);



-----------------------------------------------------------
-- Create the notes table.
--
-- Please observe the highly constrained data model. 
-- These DB-checks allow us to save code in the TCL code
-- and to write meaningful reports, because they help us 
-- to avoid garbage in the DB.
create table notes (

	-- The object_id: The note_id references acs_objects.object_id
	-- This ensures that there is an entry in acs_objects with the
	-- basic information about the object that is necessary for
	-- object permissions, search engine etc.
	note_id		integer
			constraint note_id_pk
			primary key
			constraint note_id_fk
			references acs_objects,

	-- The type of the note. Please note that we make this field
	-- obligatory (not null) and that we check the type.
	note_type	varchar(50)
			constraint note_type_nn
			not null
			constraint note_type_ck
			check (note_type in (
			    'web', 'email', 'tel', 
			    'fax', 'cell', 'address', 
			    'other'
			)),

	note		text
			constraint note_note_nn
			not null,
	
	-- We want to attach a note to another object.
	-- This way each note is associated with exactly one
	-- object.
	object_id	integer
			constraint im_object_id_fk
			references acs_objects
);

-- Allow for quick searching of all notes per object.
create index notes_object_idx on notes(object_id);

-- Avoid duplicate entries
create unique index notes_object_note_idx on notes(object_id, note);




-----------------------------------------------------------
-- Create, Drop and Name Plpg/SQL functions
--
-- These PlPg/SQL API functions represent crator/destructor
-- functions for the OpenACS object system.

-- Determine the "name" of a note.
create or replace function note__name(integer)
returns varchar as '
DECLARE
	p_note_id		alias for $1;
	v_name			text;
BEGIN
	select	note into v_name
	from	notes
	where	note_id = p_note_id;
	return v_name;
end;' language 'plpgsql';


create or replace function note__new (
	integer, varchar, timestamptz,
	integer, varchar, integer,
	varchar, integer, varchar 
) returns integer as '
DECLARE
	p_note_id	alias for $1;		-- note_id  default null
	p_object_type   alias for $2;		-- object_type default ''note''
	p_creation_date alias for $3;		-- creation_date default now()
	p_creation_user alias for $4;		-- creation_user default null
	p_creation_ip   alias for $5;		-- creation_ip default null
	p_context_id	alias for $6;		-- context_id default null

	p_note		alias for $7;		-- note_name
	p_object_id	alias for $8;		-- object_id
	p_note_type	alias for $9;		

	v_note_id	integer;
BEGIN
	v_note_id := acs_object__new (
		p_note_id,		-- object_id
		p_object_type,		-- object_type
		p_creation_date,	-- creation_date
		p_creation_user,	-- creation_user
		p_creation_ip,		-- creation_ip
		p_context_id,		-- context_id
		''t''			-- security_inherit_p
	);

	insert into notes (note_id, note, object_id, note_type) 
	values (v_note_id, p_note, p_object_id, p_note_type);

	return v_note_id;
END;' language 'plpgsql';


create or replace function note__delete(integer)
returns integer as '
DECLARE
	p_note_id	alias for $1;
BEGIN
	-- Delete any data related to the object
	delete from notes where note_id = p_note_id;

	-- Finally delete the object iself
	PERFORM acs_object__delete(p_note_id);

	return 0;
end;' language 'plpgsql';

