
<html>
<head>
<title>Array Operations</title>
<LINK REL=STYLESHEET HREF="tcl_manual.css" TYPE="text/css">
</head>
<body bgcolor=white text=black>

<h2>Array Operations</h2>

part of <a href="index.adp">Tcl for Web Nerds</a> 
by 
<a href="http://www-swiss.ai.mit.edu/~hal/hal.html">Hal Abelson</a>,
<a href="http://philip.greenspun.com/">Philip Greenspun</a>,
and
<a href="http://sandon.org/l/">Lydia Sandon</a>


<hr>


Tcl arrays are actually hash tables and have nothing in common with
the data structures called arrays in other programming languages .  A
Tcl array provides a rapid answer to the question "is there a value
associated with this key".  Here is a rat-simple example:

<blockquote>
<code><pre>
% set numeric_day(Sunday) 0
0
% set numeric_day(Monday) 1
1
% set numeric_day(Tuesday) 2
2
% # pull one value out of the hash table
% set numeric_day(Monday)
1
% # let's ask Tcl what keys are defined in the hash table
% array names numeric_day
Monday Sunday Tuesday
% # let's see if there are values for Sunday and Wednesday
% info exists numeric_day(Sunday)
1
% info exists numeric_day(Wednesday)
0
</pre></code>
</blockquote>


You don't have to declare to Tcl that you're going to treat a
particular variable as an array; just start setting variables with the
form "variable_name(key)".


<h3>You <em>Can</em> Use Tcl Array with Numbers as Keys</h3>

Here's a procedure that computes Fibonacci numbers in linear time by
storing intermediate values in an array called <code>fibvals</code>.
It uses the <code>for</code> loop, which we'll see
again in the section on <a href="control-structure.adp">control structure</a>.

<blockquote>
<code><pre>
proc fib {n} {
    set fibvals(0) 0
    set fibvals(1) 1
    for {set i 2} {$i <= $n} {incr i} {
	set fibvals($i) [expr $fibvals([expr $i - 1]) + $fibvals([expr $i - 2])]
    }
    return $fibvals($n)
}
</pre></code>
</blockquote>


<h3>Dealing with spaces inside your keys</h3>

If your index contains spaces, it will confuse the Tcl parser .  For
example, imagine an array called <code>snappy_response</code> that
contains appropriate responses to various insults, which are used as
the indices to the array.  Suppose you want to store a response for
"Have you gained weight?".  You can't feed this to Tcl as

<blockquote>
<code><pre>
set snappy_response(Have you gained weight?) "Your mama is so fat when
she goes to beach little kids shout out 'Free Willy'!"
</pre></code>
</blockquote>


Alternatives that work:

<UL>
<LI>Escape all the spaces with backslash: <BR>
<code>set snappy_response(Have\ you\ gained\ weight?) "Your mama..."</code>
<LI>Enclose the array name and parenthesized key in curly braces:<BR>
<code>set {snappy_response(Have you gained weight?)} "Your mama..."</code>
<LI>Name the index with a variable and then use the variable:
<BR>
<code>set this_insult "Have you gained weight?" <BR>
set snappy_response($this_insult) "Your mama..."
</code>
</UL>

<blockquote>
<code><pre>
% set {snappy_response(Have you gained weight?)}
Your mama is so fat when she goes to beach little kids shout out 'Free Willy'!
</pre></code>
</blockquote>



<h3>How We Actually Use Tcl Arrays:  Caching</h3>

One of the nice things about AOLserver is that it is a single Unix
process.  Thus it is easy for the result of an expensive computation
to be cached for later use by another thread.  Here is an extremely
powerful procedure that enables a programmer to cache the result of
executing any Tcl statement:

<blockquote>
<code><pre>
proc memoize {tcl_statement} {
    # tell AOLserver that this variable is to be shared among threads
    ns_share generic_cache

    # we look up the statement in the cache to see if it has already
    # been eval'd.  The statement itself is the key
    if { ![info exists generic_cache($tcl_statement)] } {
	# not in the cache already
	set statement_value [eval $tcl_statement]
	set generic_cache($tcl_statement) $statement_value
    }
    return $generic_cache($tcl_statement)
}
</pre></code>
</blockquote>


This first time this procedure is called with a particular argument,
the Tcl statement is evaluated (using Tcl's built-in <code>eval</code>
command).  The result of that evaluation is then stored in the array
variable <code>generic_cache</code> with a key consisting of the full
Tcl statement.  The next time <code>memoize</code> is called with the
same argument, the <code>info exists
generic_cache($tcl_statement)</code> will evaluate to true and the
value will be returned from the cache.

<p>

Here's how a piece of code might look before:

<blockquote>
<code><pre>
ns_return 200 text/html [page_with_top_10_popular_items]
</pre></code>
</blockquote>


If someone notices that (1) <code>page_with_top_10_popular_items</code>
requires sweeping the database and takes 30 seconds to execute, and
(2) the result doesn't change more than once or twice a day, the
natural conclusion is memoization:

<blockquote>
<code><pre>
ns_return 200 text/html [memoize "page_with_top_10_popular_items"]
</pre></code>
</blockquote>


<p class="marginnote">

Our actual toollkit contains Memoize and Memoize_for_Awhile, the
latter of which takes an argument of after how many seconds the
information in the cache should be considered stale and reevaluated.


<h3>How We Actually Use Tcl Arrays:  In-Memory Database</h3>

Typically on the Web the last thing that you'd want is an in-memory
database.  If the server crashes or the user gets bounced to another
machine by a load-balancer, you don't want critical data to be trapped
inside a Web server's virtual memory.  However, there is one situation
where you would want an in-memory database:  to store information
about the server itself.

<p>

In the ArsDigita Community System, an attempt is made to document
every externally-called procedure.  We want to build up a
documentation database that grows as procedures are defined on a
running server.  The fundamental mechanism is to define procedures
using our own procedure, <code>proc_doc</code>.  This takes a
documentation string as an extra argument, calls
<code>proc</code> to actually define the procedure, then records in a
Tcl array variable the file from which the procedure definition was
read and the doc string:

<blockquote>
<code><pre>
proc proc_doc {name args doc_string body} {
    ns_share proc_doc
    ns_share proc_source_file
    # let's define the procedure first
    proc $name $args $body
    set proc_doc($name) $doc_string
    set proc_source_file($name) [info script]
}
</pre></code>
</blockquote>


The end-result?  <a
href="http://photo.net/doc/procs.tcl">http://photo.net/doc/procs.tcl</a>. 


<h3>Full Documentation</h3>

Tcl provides support for iterating through the indices, and for
coverting lists to arrays.  These are documented in 

<a href="http://www.scriptics.com/man/tcl7.5/TclCmd/array.n.html">http://www.scriptics.com/man/tcl7.5/TclCmd/array.n.html</a>


<h3>ns_set instead</h3>

If you're using AOLserver and need to associate keys with values, you
might be better off using the ns_set data structure.  The advantages
of ns_set over Tcl arrays are the following:

<ul>
<li>you can easily pass ns_sets from procedure to procedure
<li>you can easily pass ns_sets from C code to Tcl and vice versa
</ul>

A disadvantage of ns_sets is that they require <i>O[n]</i> time to
look up the value of key, compared to <i>O[1}</i> time for the Tcl
array, which, as noted above, is actually a hash table.  If you are
managing thousands of keys, this might become significant.  Otherwise,
program using whichever data structure seems more natural.

<p>

See the Tcl Developer's guide at <a
href="http://www.aolserver.com">www.aolserver.com</a> for
documentation of the ns_set facility.



<p>

<BR><BR>
Continue on to <A HREF="numbers.adp">Numbers</A>.




<hr>
Return to <a href="index.adp">Table of Contents</a>
<br>
<br>
<address><a href="mailto:lsandon@alum.mit.edu">lsandon@alum.mit.edu</a></address>

<center><h3>Reader's Comments</h3></center>
	<blockquote>
		Another way to pass key/value pairs , especially between threads is to use nsv_set etc

		<br><br>

-- <A HREF="/shared/community-member?user_id=19369">Jamie Ross</a>, January 12, 2005</blockquote>

	<center><a href="/comments/add?page_id=4732">Add a comment</a> | <a href="/links/add?page_id=4732">Add a link</a></center>

</body>
</html>


